/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl;

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.util.ee.deploy.api.deployable.metadata.DeployableMetadataException;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;
import org.ow2.util.scan.impl.ASMScannerImpl;

/**
 * AbsDeployableMetadataFactory.
 * Provide some utility method to metadata factories.
 * @author Gael Lalire
 */
public abstract class AbsDeployableMetadataFactory {

    /**
     * The class scanner.
     */
    private IScanner scanner;

    /**
     * Default constructor (use {@link ASMScannerImpl}).
     */
    public AbsDeployableMetadataFactory() {
        this(new ASMScannerImpl());
    }

    /**
     * @param scanner the scanner to use
     */
    public AbsDeployableMetadataFactory(final IScanner scanner) {
        this.scanner = scanner;
    }

    /**
     * @param className the class name
     * @param stringSet a string set
     */
    protected void addClassToSetIfNotNull(final String className, final Set<String> stringSet) {
        if (className != null) {
            stringSet.add(className.replace('.', '/'));
        }
    }

    /**
     * @param initialClassNameSet class name found in all DD
     * @param archiveClassLocator class locator for current archive to scan
     * @param externClassLocator class locator list for extern libraries
     *        (accessibles classes which are not in archive)
     * @param archiveConfigurator a configurator (provide metadata creation
     *        method and list of annotation to analyze)
     * @throws DeployableMetadataException if error occurs
     */
    protected void scan(final Set<String> initialClassNameSet, final IClassesLocator archiveClassLocator,
            final List<? extends IClassesLocator> externClassLocator, final IArchiveConfigurator archiveConfigurator)
            throws DeployableMetadataException {
        List<IArchiveConfigurator> archiveConfiguratorList = Arrays.asList(archiveConfigurator);
        Set<String> classNameSet = new HashSet<String>();
        List<String> scannedClasses = new ArrayList<String>();
        List<String> nextClassNameToScan = new ArrayList<String>();
        nextClassNameToScan.addAll(initialClassNameSet);

        try {
            while (nextClassNameToScan.size() != 0) {
                for (String className : nextClassNameToScan) {
                    URL url = archiveClassLocator.getClassURL(className);
                    if (url == null && externClassLocator != null) {
                        for (IClassesLocator classesLocator : externClassLocator) {
                            url = classesLocator.getClassURL(className);
                            if (url != null) {
                                break;
                            }
                        }
                    }
                    if (url != null) {
                        scanner.scanClass(url, classNameSet, archiveConfiguratorList);
                    }
                    // we add to scanned even if there was no scan (class not
                    // found)
                    scannedClasses.add(className);
                }

                // keep what is not scanned
                nextClassNameToScan.clear();
                for (String className : classNameSet) {
                    if (!scannedClasses.contains(className)) {
                        nextClassNameToScan.add(className);
                    }
                }

                // recalculate classNameSet
                classNameSet.clear();
            }
        } catch (ScanException e) {
            throw new DeployableMetadataException("Error during scan", e);
        }
    }


}
