/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.configurator.visitor;

import org.ow2.util.ee.metadata.common.api.interfaces.IEjbEJB;
import org.ow2.util.ee.metadata.common.impl.struct.JEjbEJB;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.ejb.EJB} annotation.
 * @param <T> An implementation of IAnnotationEJB interface.
 * @author Florent Benoit
 */
public class JavaxEjbEJBVisitor<T extends IEjbEJB>
          extends AbsAnnotationVisitor<T> implements AnnotationType {

    /**
     * Name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * Bean interface attribute of the annotation.
     */
    private static final String BEAN_INTERFACE = "beanInterface";

    /**
     * Bean name attribute of the annotation.
     */
    private static final String BEAN_NAME = "beanName";

    /**
     * Mapped name attribute of the annotation.
     */
    private static final String MAPPED_NAME = "mappedName";
    
    /**
     * Lookup attribute of the annotation.
     */
    private static final String LOOKUP = "lookup";

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/EJB;";

    /**
     * Internal object used representing &#64;{@link javax.ejb.EJB} annotation.
     */
    private JEjbEJB jEjbEJB = null;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method metadata
     */
    public JavaxEjbEJBVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        jEjbEJB = new JEjbEJB();
    }


    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (name.equals(NAME)) {
            jEjbEJB.setName((String) value);
        } else if (BEAN_INTERFACE.equals(name)) {
            IType type = (IType) value;
            jEjbEJB.setBeanInterface(type.getClassName());
        } else if (BEAN_NAME.equals(name)) {
            jEjbEJB.setBeanName((String) value);
        } else if (MAPPED_NAME.equals(name)) {
            jEjbEJB.setMappedName((String) value);
        } else if (LOOKUP.equals(name)) {
            jEjbEJB.setLookup((String) value);
        }
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // add object
        getAnnotationMetadata().setJEjbEJB(jEjbEJB);
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }


    /**
     * @return Internal object used representing &#64;{@link javax.ejb.EJB} annotation.
     */
    protected JEjbEJB getJEjbEJB() {
        return jEjbEJB;
    }

    /**
     * Sets the jEjbEJB object.
     * @param jEjbEJB the object which replaced the previous one.
     */
    protected void setJEjbEJB(final JEjbEJB jEjbEJB) {
        this.jEjbEJB = jEjbEJB;
    }

}
