/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.configurator.visitor;

import org.ow2.util.ee.metadata.common.api.interfaces.IPersistenceUnit;
import org.ow2.util.ee.metadata.common.impl.struct.JavaxPersistenceUnit;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.persistence.PersistenceUnit}
 * annotation.
 * @param <T> An implementation of IPersistenceUnit interface.
 * @author Florent Benoit
 */
public class JavaxPersistencePersistenceUnitVisitor<T extends IPersistenceUnit> extends AbsAnnotationVisitor<T> implements
        AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/persistence/PersistenceUnit;";

    /**
     * name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * UnitName attribute of the annotation.
     */
    private static final String UNIT_NAME = "unitName";

    /**
     * Persistence context information.
     */
    private JavaxPersistenceUnit javaxPersistenceUnit = null;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method or field metadata
     */
    public JavaxPersistencePersistenceUnitVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        javaxPersistenceUnit = new JavaxPersistenceUnit();
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (NAME.equals(name)) {
            javaxPersistenceUnit.setName((String) value);
        } else if (name.equals(UNIT_NAME)) {
            javaxPersistenceUnit.setUnitName((String) value);
        }
    }

    /**
     * @return Internal object used representing &#64;{@link javax.persistence.PersistenceUnit} annotation.
     */
    protected JavaxPersistenceUnit getJavaxPersistenceUnit() {
        return javaxPersistenceUnit;
    }

    /**
     * Sets the javaxPersistenceUnit object.
     * @param javaxPersistenceUnit the object which replaced the previous one.
     */
    protected void setjavaxPersistenceUnit(final JavaxPersistenceUnit javaxPersistenceUnit) {
        this.javaxPersistenceUnit = javaxPersistenceUnit;
    }


    /**
     * Visits the end of the annotation.<br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // set flag on field
        getAnnotationMetadata().setJavaxPersistenceUnit(javaxPersistenceUnit);
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

}
