/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.configurator.visitor;

import org.ow2.util.ee.metadata.common.api.interfaces.IWebServiceRef;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.struct.JaxwsWebServiceRef;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.xml.ws.WebServiceRef}
 * annotation.
 * @param <T> An implementation of IWebServiceRef interface.
 * @author Guillaume Sauthier
 */
public class JavaxXmlWsWebServiceRefVisitor<T extends IWebServiceRef> extends AbsAnnotationVisitor<T> {

    /**
     * 'name' attribute of the annotation.
     */
    private static final String ATTR_NAME = "name";

    /**
     * 'mappedName' attribute of the annotation.
     */
    private static final String ATTR_MAPPED_NAME = "mappedName";

    /**
     * 'value' attribute of the annotation.
     */
    private static final String ATTR_VALUE = "value";

    /**
     * 'wsdlLocation' attribute of the annotation.
     */
    private static final String ATTR_WSDL_LOCATION = "wsdlLocation";

    /**
     * 'type' attribute of the annotation.
     */
    private static final String ATTR_TYPE = "type";

    /**
     * Data structure.
     */
    private IJaxwsWebServiceRef webServiceRef = null;

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/xml/ws/WebServiceRef;";

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method metadata
     */
    public JavaxXmlWsWebServiceRefVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        webServiceRef = new JaxwsWebServiceRef();
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (name.equals(ATTR_NAME)) {
            webServiceRef.setName((String) value);
        } else if (name.equals(ATTR_TYPE)) {
            IType type = (IType) value;
            webServiceRef.setType(type.getClassName());
        } else if (name.equals(ATTR_MAPPED_NAME)) {
            // mapped-name is not useful for WS
            webServiceRef.setMappedName(((String) value));
        } else if (name.equals(ATTR_VALUE)) {
            IType type = (IType) value;
            webServiceRef.setValue(type.getClassName());
        } else if (name.equals(ATTR_WSDL_LOCATION)) {
            webServiceRef.setWsdlLocation((String) value);
        }
    }

    @Override
    public void visitEnd() {
        getAnnotationMetadata().setJaxwsWebServiceRef(webServiceRef);
    }

    /**
     * @return Internal object used representing &#64;{@link javax.xml.ws.WebServiceRef} annotation.
     */
    protected IJaxwsWebServiceRef getJaxwsWebServiceRef() {
        return this.webServiceRef;
    }

    /**
     * Sets the IJaxwsWebServiceRef object.
     * @param webServiceRef the object which replaced the previous one.
     */
    protected void setJaxwsWebServiceRef(final IJaxwsWebServiceRef webServiceRef) {
        this.webServiceRef = webServiceRef;
    }

    public String getType() {
        return TYPE;
    }
}
