/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.configurator.visitor.clazz;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.ICommonClassMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxXmlWsWebServiceRefVisitor;
import org.ow2.util.ee.metadata.common.impl.struct.JaxwsWebServiceRef;

/**
 * This class manages the handling of &#64;{@link javax.xml.ws.WebServiceRefs}
 * annotation.
 * @author Guillaume Sauthier
 */
public class JavaxXmlWsWebServiceRefsVisitor extends
        JavaxXmlWsWebServiceRefVisitor<ICommonClassMetadata<?, ?, ?>> {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/xml/ws/WebServiceRefs;";

    /**
     * List of JaxwsWebServiceRefs object.
     */
    private List<IJaxwsWebServiceRef> jaxwsWebServiceRefs = null;

    /**
     * Object is added to the list.
     */
    private boolean isAdded = false;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class metadata
     */
    public JavaxXmlWsWebServiceRefsVisitor(final ICommonClassMetadata<?, ?, ?> annotationMetadata) {
        super(annotationMetadata);
        jaxwsWebServiceRefs = new ArrayList<IJaxwsWebServiceRef>();
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        // list not empty, need to create another reference
        // at the first item found
        if (jaxwsWebServiceRefs.size() > 0 && isAdded) {
            setJaxwsWebServiceRef(new JaxwsWebServiceRef());
            isAdded = false;
        }

        // do super code
        super.visit(name, value);
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // add object in the list
        if (!isAdded) {
            jaxwsWebServiceRefs.add(getJaxwsWebServiceRef());
            isAdded = true;
        }

        // update list
        getAnnotationMetadata().setJaxwsWebServiceRefs(jaxwsWebServiceRefs);
    }

    /**
     * @return type of the annotation (its description)
     */
    @Override
    public String getType() {
        return TYPE;
    }

}
